@@ -1,6 +1,6 @@
-/* 
+/*
     TiMidity++ -- MIDI to WAVE converter and player
-    Copyright (C) 1999-2002 Masanao Izumo <mo@goice.co.jp>
+    Copyright (C) 1999-2004 Masanao Izumo <iz@onicos.co.jp>
     Copyright (C) 1995 Tuukka Toivonen <tt@cgs.fi>
 
     This program is free software; you can redistribute it and/or modify
@@ -15,11 +15,11 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-	
-	Macintosh interface for TiMidity
-	by T.Nogami	<t-nogami@happy.email.ne.jp>
-	
+    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
+
+    Macintosh interface for TiMidity
+    by T.Nogami <t-nogami@happy.email.ne.jp>
+
     mac_c.c
     Macintosh control mode
 */
@@ -85,12 +85,14 @@ static void ctl_event(CtlEvent *e);
 ControlMode ctl= 
 {
   "mac interface", 'm',
+  "mac",
   1,1,0,
   0,
   ctl_open,
   ctl_close,
   ctl_pass_playing_list,
   ctl_read,
+  NULL,
   cmsg,
   ctl_event
 };
@@ -553,11 +555,16 @@ void change_ListRow( short row, const MidiFile* file)
 	char	buf[256]="",*p;
 	
 	if( file && file->mfn && file->mfn->title )
-		strcat(buf, file->mfn->title);
+	{
+		strncpy(buf, file->mfn->title, sizeof(buf) - 1);
+		buf[sizeof(buf)-1] = '\0';
+	}
 	
 	p= strrchr(file->filename, PATH_SEP);
 	if( p ){
-		strcat(buf, "  ("); strcat(buf, p+1); strcat(buf, ")");
+		size_t len = strlen(buf);
+		snprintf(&buf[len], sizeof(buf)-len-1, "  (%s)", p+1);
+		buf[sizeof(buf)-1] = '\0';
 	}
 
 	SetPt(&aCell, 0, row);
@@ -739,10 +746,14 @@ static int	message_DocWin(int message, long param)
 			char		*p, docname[256];
 
 			//midiname= (char*)param;
-		    strcpy(docname, midiname);
+		    strncpy(docname, midiname, sizeof docname - 1);
+		    docname[sizeof docname - 1] = '\0';
 	    	if((p = strrchr(docname, '.')) == NULL){
 				return 0;
 		    }
+		    else if(p - docname >= sizeof docname - 4) {
+				return 0;	/* cannot strcpy: that cause buffer overrun */
+		    }
 		    if('A' <= p[1] && p[1] <= 'Z')	strcpy(p + 1, "DOC");
 	    						else		strcpy(p + 1, "doc");
 
