@@ -46,6 +46,7 @@
 #include "tables.h"
 #include "filter.h"
 #include "quantity.h"
+#include "freq.h"
 
 #define INSTRUMENT_HASH_SIZE 128
 struct InstrumentCache
@@ -661,7 +662,7 @@ static Instrument *load_gus_instrument(char *name,
 			return ip;
 		}
 	/* Open patch file */
-	if (! (tf = open_file(name, 2, OF_NORMAL))) {
+	if (! (tf = open_file_r(name, 2, OF_NORMAL))) {
 #ifdef PATCH_EXT_LIST
 		int name_len, ext_len;
 		static char *patch_ext[] = PATCH_EXT_LIST;
@@ -679,7 +680,7 @@ static Instrument *load_gus_instrument(char *name,
 					continue;	/* duplicated ext. */
 				strcpy((char *) tmp, name);
 				strcat((char *) tmp, patch_ext[i]);
-				if ((tf = open_file((char *) tmp, 1, OF_NORMAL))) {
+				if ((tf = open_file_r((char *) tmp, 1, OF_NORMAL))) {
 					noluck = 0;
 					break;
 				}
@@ -988,6 +989,17 @@ fail:
 		 */
 		if (sp->note_to_use && ! (sp->modes & MODES_LOOPING))
 			pre_resample(sp);
+
+		/* do pitch detection on drums if surround chorus is used */
+		if (dr && opt_surround_chorus)
+		{
+		    sp->chord = -1;
+		    sp->root_freq_detected = freq_fourier(sp, &(sp->chord));
+		    sp->transpose_detected =
+			assign_pitch_to_freq(sp->root_freq_detected) -
+			assign_pitch_to_freq(sp->root_freq / 1024.0);
+		}
+
 #ifdef LOOKUP_HACK
 		squash_sample_16to8(sp);
 #endif
@@ -1477,6 +1489,12 @@ void free_instruments(int reload_default_inst)
 		    free_instrument(ip);
 		bank->tone[j].instrument = NULL;
 	    }
+#if 0
+		if ((drumset[i] != NULL) && (drumset[i]->alt != NULL)) {
+			free(drumset[i]->alt);
+			drumset[i]->alt = NULL;
+		}
+#endif
     }
 
     /* Free GUS/patch instruments */
