@@ -1,34 +1,33 @@
 /*
     TiMidity++ -- MIDI to WAVE converter and player
-    Copyright (C) 1999-2002 Masanao Izumo <mo@goice.co.jp>
+    Copyright (C) 1999-2005 Masanao Izumo <iz@onicos.co.jp>
     Copyright (C) 1995 Tuukka Toivonen <tt@cgs.fi>
+
+    This program is free software; you can redistribute it and/or modify
+    it under the terms of the GNU General Public License as published by
+    the Free Software Foundation; either version 2 of the License, or
+    (at your option) any later version.
+
+    This program is distributed in the hope that it will be useful,
+    but WITHOUT ANY WARRANTY; without even the implied warranty of
+    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+    GNU General Public License for more details.
+
+    You should have received a copy of the GNU General Public License
+    along with this program; if not, write to the Free Software
+    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
+
+    This code from awesfx
+    Modified by Masanao Izumo <mo@goice.co.jp>
+
+    ================================================================
+    parsesf.c
+         parse SoundFont layers and convert it to AWE driver patch
+
+    Copyright (C) 1996,1997 Takashi Iwai
+    ================================================================
 */
 
-/* This code from awesfx
- * Modified by Masanao Izumo <mo@goice.co.jp>
- */
-
-/*================================================================
- * parsesf.c
- *	parse SoundFonr layers and convert it to AWE driver patch
- *
- * Copyright (C) 1996,1997 Takashi Iwai
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
- *================================================================*/
-
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif /* HAVE_CONFIG_H */
@@ -55,6 +54,7 @@
 #include "output.h"
 #include "filter.h"
 #include "resample.h"
+#include "interface.h"
 
 #define FILENAME_NORMALIZE(fname) url_expand_home_dir(fname)
 #define FILENAME_REDUCED(fname)   url_unexpand_home_dir(fname)
@@ -66,11 +66,12 @@
  *----------------------------------------------------------------*/
 
 #ifdef CFG_FOR_SF
+#define SF_CLOSE_EACH_FILE 0
 #define SF_SUPPRESS_ENVELOPE
 #define SF_SUPPRESS_TREMOLO
 #define SF_SUPPRESS_VIBRATO
 #else
-#define SF_CLOSE_EACH_FILE
+#define SF_CLOSE_EACH_FILE 1
 /*#define SF_SUPPRESS_ENVELOPE*/
 /*#define SF_SUPPRESS_TREMOLO*/
 /*#define SF_SUPPRESS_VIBRATO*/
@@ -209,6 +210,7 @@ static void convert_vibrato(SampleList *vp, LayerTable *tbl);
 
 /*----------------------------------------------------------------*/
 
+int opt_sf_close_each_file = SF_CLOSE_EACH_FILE;
 static SFInsts *sfrecs = NULL;
 static SFInsts *current_sfrec = NULL;
 #define def_drum_inst 0
@@ -274,6 +276,21 @@ void remove_soundfont(char *sf_file)
 	end_soundfont(sf);
 }
 
+void free_soundfonts()
+{
+	SFInsts *sf, *next;
+	
+	for (sf = sfrecs; sf != NULL; sf = next) {
+		if ((sf->tf != NULL) && (sf->tf->url != NULL))
+			free(sf->tf->url);
+		if (sf->tf != NULL)
+			free(sf->tf);
+		reuse_mblock(&sf->pool);
+		next = sf->next;
+		free(sf);
+	}
+}
+
 char *soundfont_preset_name(int bank, int preset, int keynote,
 			    char **sndfile)
 {
@@ -352,12 +369,14 @@ static void init_sf(SFInsts *rec)
 		(char *)SFStrdup(rec, sfinfo.preset[i].hdr.name);
 	free_soundfont(&sfinfo);
 
-#ifndef SF_CLOSE_EACH_FILE
-	if(!IS_URL_SEEK_SAFE(rec->tf->url))
-#endif
-	{
-	    close_file(rec->tf);
-	    rec->tf = NULL;
+	if (! opt_sf_close_each_file) {
+		if (! IS_URL_SEEK_SAFE(rec->tf->url)) {
+			close_file(rec->tf);
+			rec->tf = NULL;
+		}
+	} else {
+		close_file(rec->tf);
+		rec->tf = NULL;
 	}
 }
 
@@ -419,10 +438,9 @@ static Instrument *try_load_soundfont(SFInsts *rec, int order, int bank,
 			end_soundfont(rec);
 			return NULL;
 		}
-#ifndef SF_CLOSE_EACH_FILE
-		if(!IS_URL_SEEK_SAFE(rec->tf->url))
-		    rec->tf->url = url_cache_open(rec->tf->url, 1);
-#endif /* SF_CLOSE_EACH_FILE */
+		if (! opt_sf_close_each_file)
+			if (! IS_URL_SEEK_SAFE(rec->tf->url))
+				rec->tf->url = url_cache_open(rec->tf->url, 1);
 	}
 
 	addr = INSTHASH(bank, preset, keynote);
@@ -436,10 +454,10 @@ static Instrument *try_load_soundfont(SFInsts *rec, int order, int bank,
 	if (ip && ip->samples)
 		inst = load_from_file(rec, ip);
 
-#ifdef SF_CLOSE_EACH_FILE
-	close_file(rec->tf);
-	rec->tf = NULL;
-#endif
+	if (opt_sf_close_each_file) {
+		close_file(rec->tf);
+		rec->tf = NULL;
+	}
 
 	return inst;
 }
@@ -664,6 +682,17 @@ static Instrument *load_from_file(SFInsts *rec, InstList *ip)
 		if (sample->note_to_use && !(sample->modes & MODES_LOOPING))
 			pre_resample(sample);
 
+		/* do pitch detection on drums if surround chorus is used */
+		if (ip->pat.bank == 128 && opt_surround_chorus)
+		{
+		    sample->chord = -1;
+		    sample->root_freq_detected =
+		    	freq_fourier(sample, &(sample->chord));
+		    sample->transpose_detected =
+			assign_pitch_to_freq(sample->root_freq_detected) -
+			assign_pitch_to_freq(sample->root_freq / 1024.0);
+		}
+
 #ifdef LOOKUP_HACK
 		squash_sample_16to8(sample);
 #endif
@@ -1003,7 +1032,7 @@ static int sanity_range(LayerTable *tbl)
  *----------------------------------------------------------------*/
 
 #ifdef CFG_FOR_SF
-int opt_reverb_control;	/* to avoid warning. */
+int opt_reverb_control, opt_surround_chorus;	/* to avoid warning. */
 static int cfg_for_sf_scan(char *name, int x_bank, int x_preset, int x_keynote_from, int x_keynote_to, int romflag);
 #endif
 
@@ -1811,6 +1840,7 @@ ControlMode w32gui_control_mode =
     ctl_close,
     dumb_pass_playing_list,
     ctl_read,
+    NULL,
     cmsg,
     ctl_event
 };
@@ -2086,11 +2116,8 @@ int main(int argc, char **argv)
 						inst = try_load_soundfont(sf, -1, 128, x_preset, x_keynote);
 						if(inst != NULL) {
 							freq = freq_fourier(inst->sample, &chord);
-							if (freq > 0) {
-								note = ceil(-36.87631656f + 17.31234049f * log(freq)); /* freq.c */
-								if (note >= LOWEST_PITCH && note <= HIGHEST_PITCH) {
-									x_playnote[x_keynote] = note;
-								}
+							if (freq != 260) { /* 260 Hz is only returned when pitch is uncertain */
+								x_playnote[x_keynote] = assign_pitch_to_freq(freq);
 							}
 							free_instrument(inst);
 						}
